@@ -133,9 +133,7 @@
 			 '(boolean s7_is_boolean s7_boolean s7_make_boolean bool)
 			 '(real s7_is_real s7_number_to_real_with_caller s7_make_real s7_double)
 
-			 ;; '(complex s7_is_complex #f s7_make_complex s7_Complex)
-			 ;; the typedef is around line 6116 in s7.c, but we also need s7_complex which requires the s7_Complex type
-			 ;; xen.h uses (s7_real_part(a) + s7_imag_part(a) * _Complex_I) instead since c++ won't let use define s7_Complex in s7.h
+			 ;; '(complex s7_is_complex s7_to_c_complex?? s7_make_complex s7_complex)
 
 			 '(string s7_is_string s7_string s7_make_string char*)
 			 (list 'character 's7_is_character 's7_character 's7_make_character (symbol "unsigned char"))
@@ -227,8 +225,8 @@
 	  (type-symbols ())
 	  (p #f)
 	  (pp (open-output-string))
-	  (int-funcs ())  ; functions guaranteed to return int
-	  (double-funcs ())  ; functions returning double, all args double
+	  (int-funcs ())        ; functions guaranteed to return int
+	  (double-funcs ())     ; functions returning double, all args double
 	  (double-int-funcs ()) ; functions return double, args are (integer double)
 	  (sig-symbols (list (cons 'integer? 0) (cons 'boolean? 0) (cons 'real?  0) (cons 'float? 0) 
 			     (cons 'char? 0) (cons 'string? 0) (cons 'c-pointer? 0) (cons 't 0)))
@@ -291,7 +289,7 @@
 	       (format p "#include <~A>~%" header))
 	     headers))
 	(format p "#include \"s7.h\"~%~%")
-	(format p "static s7_pointer c_pointer_string, string_string, character_string, boolean_string, real_string, complex_string, integer_string;~%"))
+	(format p "static s7_pointer fsym, s7_F, s7_unspec, ffunc, c_pointer_string, string_string, character_string, boolean_string, real_string, complex_string, integer_string;~%"))
       
       (define collides?
 	(let ((all-names (hash-table)))
@@ -414,18 +412,19 @@
 		  (format pp ")")
 		  
 		  (if (eq? return-translator 's7_make_c_pointer_with_type)
-		      (format pp ", ~S, s7_f(sc))" (type->type-symbol return-type))
+		      (format pp ", ~S, s7_F)" (type->type-symbol return-type))
 		      (if (symbol? return-translator)
 			  (format pp ")")))
 		  (format pp (if (not (eq? return-translator #t))
 				 ");~%"
-				 ";~%  return(s7_unspecified(sc));~%"))
+				 ";~%  return(s7_unspec);~%"))
 		  (format pp "}~%"))
 		
 		;; add optimizer connection
 		(when (and (eq? return-type 'double)
 			   (< num-args 5)
-			   (sig-every? (lambda (p) (eq? p 'double)) arg-types))
+			   (sig-every? (lambda (p) (eq? p 'double)) arg-types)
+			   (not (defined? (symbol scheme-name) (rootlet)))) ; see below, double-funcs entry not used if already defined
 		  (let ((local-name #f))
 		    (case num-args
 		      ((0)
@@ -449,8 +448,9 @@
 		       (format pp "static s7_double ~A~A(s7_double x1, s7_double x2, s7_double x3, s7_double x4) {return(~A(x1, x2, x3, x4));}~%"
 			       func-name local-name func-name)))
 		    (set! double-funcs (cons (list func-name scheme-name local-name) double-funcs))))
-		
+
 		(when (and (memq return-type '(int size_t))        ; int (f int|double|void)
+			   (not (defined? (symbol scheme-name) (rootlet))) ; see below, int-funcs entry not used if already defined
 			   (or ;(= num-args 0)
 			    (and (= num-args 1)
 				 (memq (car arg-types) '(int size_t double)))
@@ -485,6 +485,7 @@
 		;; other possibilities: d_7pi|pii p=double* etc piid=checks in s7 (assumes float-vector)
 		;;   d_pd [lots of d_pdd, d_p, p_i and i_p]
 		;;   but how to recognize the "p" portions? (d_7pi with p="s7_pointer" gets no hits in libgsl)
+		;; libgsl: d_i: 15, d_ii: 5, [handled: d_id: 32], d_iid: 7, d_ddi: 4, d_idd: 10, d_idi: 6, i_ddd i_iii: 1, i_dddd: 4
 		
 		(format pp "~%")
 		(set! functions (cons (list scheme-name base-name 
@@ -558,6 +559,8 @@
 	   signatures)
 	  (format p "  }~%~%"))
 
+	(format p "  s7_F = s7_f(sc);~%")
+	(format p "  s7_unspec = s7_unspecified(sc);~%")
 	(format p "  string_string = s7_make_semipermanent_string(sc, \"a string\");~%")
 	(format p "  c_pointer_string = s7_make_semipermanent_string(sc, \"a c-pointer\");~%")
 	(format p "  character_string = s7_make_semipermanent_string(sc, \"a character\");~%")
@@ -591,7 +594,7 @@
 		    (scheme-name (string-append prefix (if (> (length prefix) 0) ":" "") c-name))
 		    (trans (C->s7 type)))
 	       (if (eq? trans 's7_make_c_pointer_with_type)
-		   (format p "  s7_define(sc, cur_env, s7_make_symbol(sc, ~S), ~A(sc, (~A)~A, s7_make_symbol(sc, ~S), s7_f(sc)));~%" 
+		   (format p "  s7_define(sc, cur_env, s7_make_symbol(sc, ~S), ~A(sc, (~A)~A, s7_make_symbol(sc, ~S), s7_F));~%" 
 			   scheme-name
 			   trans
 			   (C->s7-cast type)
@@ -615,7 +618,7 @@
 		    (trans (C->s7 type)))
 	       (format p "#ifdef ~A~%" c-name)
 	       (if (eq? trans 's7_make_c_pointer_with_type)
-		   (format p "  s7_define(sc, cur_env, s7_make_symbol(sc, ~S), s7_make_c_pointer_with_type(sc, (~A)~A, s7_make_symbol(sc, \"~S\"), s7_f(sc)));~%" 
+		   (format p "  s7_define(sc, cur_env, s7_make_symbol(sc, ~S), s7_make_c_pointer_with_type(sc, (~A)~A, s7_make_symbol(sc, \"~S\"), s7_F));~%" 
 			   scheme-name
 			   (C->s7-cast type)
 			   c-name
@@ -637,14 +640,15 @@
 		 (num-args    (f 3))
 		 (opt-args    (if (> (length f) 4) (f 4) 0))
 		 (sig         (and (> (length f) 5) (f 5))))
-	     (format p "~%  s7_define(sc, cur_env,~%            s7_make_symbol(sc, ~S),~%" scheme-name)
-	     (format p "            s7_make_typed_function(sc, ~S, ~A, ~D, ~D, false, ~S, ~A));~%"
+	     (format p "~%  s7_define(sc, cur_env,~%            fsym = s7_make_symbol(sc, ~S),~%" scheme-name)
+	     (format p "            ffunc = s7_make_typed_function_with_environment(sc, ~S, ~A, ~D, ~D, false, ~S, ~A, cur_env));~%"
 		     scheme-name
 		     base-name
 		     num-args
 		     opt-args
 		     helpf
-		     (if (pair? sig) (signatures sig) 'NULL))))
+		     (if (pair? sig) (signatures sig) 'NULL))
+	     (format p "  s7_symbol_set_initial_value(sc, fsym, ffunc);~%"))) ; not sure this is a good idea, added 24-May-24
 	 functions)
 	
 	;; optimizer connection
@@ -685,7 +689,8 @@
 	       ;; I assume the caller is also compiled with these flags?
 	       (system (format #f "~A -c ~A -o ~A ~A ~A" 
 			       *cload-c-compiler* c-file-name o-file-name *cload-cflags* cflags))
-	       (system (format #f "~A ~A -o ~A -dynamic -bundle -undefined suppress -flat_namespace ~A ~A" 
+	       (system (format #f "~A ~A s7.o -o ~A -dynamic -bundle -flat_namespace ~A ~A"  
+			       ;; -undefined suppress is deprecated, so trying adding s7.o
 			       *cload-c-compiler* o-file-name so-file-name *cload-ldflags* ldflags)))
 	      
 	      ((provided? 'freebsd)
@@ -765,7 +770,7 @@
       (unless (and output-name
 		   (file-exists? c-file-name)
 		   (file-exists? so-file-name)
-		   (provided? 'system-extras)
+		   (provided? 'system-extras) ; too late!
 		   (>= (file-mtime so-file-name) (file-mtime c-file-name))
 		   (not (and (file-exists? (port-filename))
 			     (< (file-mtime so-file-name) (file-mtime (port-filename))))))
